#include <cxxtest/TestSuite.h>

#include "String.h"

class StringTestSuite : public CxxTest::TestSuite
{
public:
	void test_ascii()
	{
		char data[] = { "hello world" };

		String string( data );

		TS_ASSERT_EQUALS( strlen( data ), strlen( string.ascii() ) );
		TS_ASSERT_SAME_DATA( data, string.ascii(), strlen( data ) );
	}

	void test_bracket_operator()
	{
	  	String myString( "red apple" );

		TS_ASSERT_EQUALS( myString[0], 'r' );
		TS_ASSERT_EQUALS( myString[3], ' ' );
		TS_ASSERT_EQUALS( myString[8], 'e' );
	}

	void test_empty_constructor()
	{
		String string;

		TS_ASSERT_EQUALS( string.length(), 0U );
	}

	void test_assignemnt_from_c_str()
	{
		String myString;

		TS_ASSERT_EQUALS( myString.length(), 0U );

		myString = "apple";

		TS_ASSERT_EQUALS( myString.length(), strlen( "apple" ) );
		TS_ASSERT_SAME_DATA( myString.ascii(), "apple", myString.length() );
	}

	void test_equality_operator()
	{
		String string( "apple" );
		String otherString( "apple" );
		String different( "banana" );

		TS_ASSERT( string == otherString );
		TS_ASSERT( !( string == different ) );
		TS_ASSERT( string != different );

		TS_ASSERT_EQUALS( string, "apple" );
		TS_ASSERT_DIFFERS( string, "banana" );

        String otherApple( "apple" );

        TS_ASSERT_EQUALS( string, otherString );
        TS_ASSERT_EQUALS( string, otherString.ascii() );
    }

    void test_constructor_limited_length()
    {
        char data[] = "hello world";

        String string( data, 6 );

        String expected( "hello " );
        TS_ASSERT_EQUALS( string, expected );
    }

    void test_comparison_operator()
    {
        TS_ASSERT( String( "a" ) < String( "b" ) );
        TS_ASSERT( String( "aa" ) < String( "ab" ) );
    }

    void test_concatenation()
    {
        String line1 = "hello";
        String line2 = " world";
        String expected = "hello world";

        TS_ASSERT_EQUALS( line1 + line2, expected );

        line1 += line2;

        TS_ASSERT_EQUALS( line1, expected );

        line1 = "hello";

        String line3 = line1 + " world";

        TS_ASSERT_EQUALS( line3, expected );

        line1 += " world";

        TS_ASSERT_EQUALS( line1, expected );
    }

    void test_tokenize()
    {
        String string = "This is a string with several    space  separated words";

        char delimiters[] = " ";
        Vector<String> tokens = string.tokenize( delimiters );

        TS_ASSERT_EQUALS( tokens.size(), 9U );

        TS_ASSERT_EQUALS( tokens[0], "This" );
        TS_ASSERT_EQUALS( tokens[1], "is" );
        TS_ASSERT_EQUALS( tokens[2], "a" );
        TS_ASSERT_EQUALS( tokens[3], "string" );
        TS_ASSERT_EQUALS( tokens[4], "with" );
        TS_ASSERT_EQUALS( tokens[5], "several" );
        TS_ASSERT_EQUALS( tokens[6], "space" );
        TS_ASSERT_EQUALS( tokens[7], "separated" );
        TS_ASSERT_EQUALS( tokens[8], "words" );

        String empty;

        tokens = empty.tokenize( delimiters );
        TS_ASSERT_EQUALS( tokens.size(), 0U );
    }

    void test_tokenize__two_delimiters()
    {
        String string = "ThisZis a stringZwith several ZZZ   space  separated words";

        char delimiters[] = " Z";
        Vector<String> tokens = string.tokenize( delimiters );

        TS_ASSERT_EQUALS( tokens.size(), 9U );

        TS_ASSERT_EQUALS( tokens[0], "This" );
        TS_ASSERT_EQUALS( tokens[1], "is" );
        TS_ASSERT_EQUALS( tokens[2], "a" );
        TS_ASSERT_EQUALS( tokens[3], "string" );
        TS_ASSERT_EQUALS( tokens[4], "with" );
        TS_ASSERT_EQUALS( tokens[5], "several" );
        TS_ASSERT_EQUALS( tokens[6], "space" );
        TS_ASSERT_EQUALS( tokens[7], "separated" );
        TS_ASSERT_EQUALS( tokens[8], "words" );
    }

    void test_contains()
    {
        String string( "This is a simple string" );

        TS_ASSERT( string.contains( "This is" ) );
        TS_ASSERT( string.contains( "a simple" ) );
        TS_ASSERT( string.contains( " string" ) );
        TS_ASSERT( string.contains( "" ) );
        TS_ASSERT( string.contains( " " ) );

        TS_ASSERT( !string.contains( "Thisis" ) );
        TS_ASSERT( !string.contains( "crazy" ) );
    }

    void test_find()
    {
        String string( "This is a simple string" );

        TS_ASSERT_EQUALS( string.find( "is a" ), 5U );
        TS_ASSERT_EQUALS( string.find( " " ), 4U );
        TS_ASSERT_EQUALS( string.find( "simple string" ), 10U );

        TS_ASSERT_EQUALS( string.find( "crazy" ), String::Super::npos );
    }

    void test_substring()
    {
        String string( "This is a simple string" );

        TS_ASSERT_EQUALS( string.substring( 0, 4 ), "This" );
        TS_ASSERT_EQUALS( string.substring( 4, 3 ), " is" );
        TS_ASSERT_EQUALS( string.substring( 7, 3 ), " a " );
    }

    void test_repalce()
    {
        String string( "As I was going to st Ives" );

        string.replace( "going", "running" );

        TS_ASSERT_EQUALS( string, String( "As I was running to st Ives" ) );

        string.replace( "I", "you" );

        TS_ASSERT_EQUALS( string, String( "As you was running to st Ives" ) );

        TS_ASSERT_THROWS_NOTHING( string.replace( "bla", "other bla" ) );

        TS_ASSERT_EQUALS( string, String( "As you was running to st Ives" ) );
    }

    void test_trim()
    {
        TS_ASSERT_EQUALS( String( "   hello      world     \n" ).trim() , "hello      world" );
        TS_ASSERT_EQUALS( String( "              \n" ).trim() , "" );
        TS_ASSERT_EQUALS( String( "   a" ).trim() , "a" );
        TS_ASSERT_EQUALS( String( "   ab\n" ).trim() , "ab" );
        TS_ASSERT_EQUALS( String( "ab\n   \n" ).trim() , "ab" );
        TS_ASSERT_EQUALS( String( "hey there   \n" ).trim() , "hey there" );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
