#include <cxxtest/TestSuite.h>

#include "Error.h"
#include "MockErrno.h"
#include "String.h"
#include "UniqueVector.h"

class UniqueVectorTestSuite : public CxxTest::TestSuite
{
public:
    MockErrno *mockErrno;

    void setUp()
    {
        TS_ASSERT( mockErrno = new MockErrno );
    }

    void tearDown()
    {
        TS_ASSERT_THROWS_NOTHING( delete mockErrno );
    }

    void test_copy_constructor_and_assignemnt()
    {
        UniqueVector<int> v1;

        v1.append( 1 );
        v1.append( 1 );
        v1.append( 2 );

        UniqueVector<int> v2 = v1;
        UniqueVector<int> v3( v1 );

        v1.erase( 1 );
        TS_ASSERT_EQUALS( v1.size(), 1U );
        TS_ASSERT_EQUALS( v2.size(), 2U );

        TS_ASSERT_EQUALS( v2[0], 1 );
        TS_ASSERT_EQUALS( v2[1], 2 );

        TS_ASSERT_EQUALS( v3.size(), 2U );

        TS_ASSERT_EQUALS( v3[0], 1 );
        TS_ASSERT_EQUALS( v3[1], 2 );
    }

    void test_append()
    {
        UniqueVector<int> vector;

        vector.append( 1 );
        vector.append( 2 );

        TS_ASSERT_EQUALS( vector.size(), 2U );

        vector.append( 1 );
        vector.append( 2 );

        TS_ASSERT_EQUALS( vector.size(), 2U );
        TS_ASSERT_EQUALS( vector[0], 1 );
        TS_ASSERT_EQUALS( vector[1], 2 );

        vector.append( 3 );
        TS_ASSERT_EQUALS( vector.size(), 3U );

        vector.append( 3 );
        TS_ASSERT_EQUALS( vector.size(), 3U );
    }

    void test_assignment()
    {
        Vector<int> a;
        a.append( 1 );
        a.append( 1 );
        a.append( 2 );
        a.append( 2 );
        a.append( 1 );

        UniqueVector<int> b;

        b = a;

        TS_ASSERT_EQUALS( b.size(), 2U );
        TS_ASSERT_EQUALS( b[0], 1 );
        TS_ASSERT_EQUALS( b[1], 2 );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
