#ifndef __HttpDispatcher_h__
#define __HttpDispatcher_h__

#include "BThread.h"
#include "IEvent.h"

class HttpDispatcher : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForRequests();
            classifyRequest();
        }
    }

private:
    void waitForRequests()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_DATA_RECEIVED );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_DATA_RECEIVED );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void classifyRequest()
    {
        String request( lastEvent().payload().asChar(), lastEvent().payloadSize() );

        printf( "Http Dispatcher: %s\n", request.ascii() );
        fflush( 0 );

        if ( request.contains( "GET" ) )
        {
            handleGetRequest();
        }
        else
        {
            generateErrorMessage();
        }
    }

    void handleGetRequest()
    {
        clearEventVectors();

        _requested.append( Event( IEvent::HTTP_GET_REQUEST, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_GET_REQUEST );

        bSync( _requested, _watched, _blocked, "Get" );
    }

    void generateErrorMessage()
    {
        clearEventVectors();

        _requested.append( IEvent::HTTP_GENERATE_ERROR );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_GENERATE_ERROR );

        bSync( _requested, _watched, _blocked, "Error" );
    }

    Vector<Event> allRelevantEventsExcept( Event event )
    {
        Vector<Event> result;

        if ( event != IEvent::HTTP_DATA_RECEIVED ) result.append( IEvent::HTTP_DATA_RECEIVED );
        if ( event != IEvent::HTTP_GET_REQUEST ) result.append( IEvent::HTTP_GET_REQUEST );

        return result;
    }
};

#endif // __HttpDispatcher_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
