#ifndef __HttpFoundHandler_h__
#define __HttpFoundHandler_h__

#include "BThread.h"
#include "File.h"
#include "IEvent.h"

class HttpFoundHandler : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForFoundRequest();
            sendResponse();
            endSession();
            doneHandlingRequest();
        }
    }

private:
    enum {
        MAX_FILE_SIZE = 4096,
    };

    void waitForFoundRequest()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_GENERATE_FOUND );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void sendResponse()
    {
        Stringf foundString( "%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
                             "HTTP/1.1 302 Found\r\n",
                             "Server: BP Server\r\n",
                             "Content-Type: text/html; charset=\%s\r\n",
                             "Location: ",
                             properLocation().ascii(),
                             "\r\nConnection: close\r\n",
                             "\r\n",
                             "<HTML>\n",
                             "<HEAD><TITLE>302 Found Found</TITLE></HEAD>\n",
                             "<BODY>\n",
                             "<H4>302 Found</H4>\n",
                             "Directories must end with a slash.\n",
                             "</BODY>\n",
                             "</HTML>\n" );

        HeapData foundMessage = ConstSimpleData( foundString.ascii(), foundString.length() );

        clearEventVectors();
        _requested.append( Event( IEvent::DATA_FROM_HTTP_TO_TCP, foundMessage ) );
        _blocked.append( IEvent::HTTP_GENERATE_FOUND );

        bSync( _requested, _watched, _blocked, "Send" );
    }

    String properLocation()
    {
        String original( lastEvent().payload().asChar(), lastEvent().payloadSize() );
        return original + "/";
    }

    void endSession()
    {
        clearEventVectors();

        _requested.append( IEvent::HTTP_END_SESSION );
        _blocked.append( IEvent::HTTP_GENERATE_FOUND );
        bSync( _requested, _watched, _blocked, "EndSession" );
    }

    void doneHandlingRequest()
    {
        clearEventVectors();

        _requested.append( IEvent::HTTP_DONE_HANDLING_REQUEST );
        _blocked.append( IEvent::HTTP_GENERATE_FOUND );
        bSync( _requested, _watched, _blocked, "Done" );
    }
};

#endif // __HttpFoundHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
