#ifndef __TcpServer_h__
#define __TcpServer_h__

#include "BProgram.h"
#include "BridgeToHttp.h"
#include "BridgeToTcp.h"
#include "HttpDispatcher.h"
#include "HttpErrorHandler.h"
#include "HttpFoundHandler.h"
#include "HttpGetHandler.h"
#include "Logger.h"
#include "RawSocketManager.h"
#include "SequenceMaster.h"
#include "TcpAckSender.h"
#include "TcpDispatcher.h"
#include "TcpFinHandler.h"
#include "TcpPushHandler.h"
#include "TcpPushHandler.h"
#include "TcpPushSender.h"
#include "TcpSynHandler.h"

class TcpServer
{
public:
    static void start()
    {
        TcpServer().run();
    }

    void run()
    {
        runEngine();

        try
        {
            runProgram();
            sleepForever();
        }
        catch ( const Error &e )
        {
            reportError( e );
        }
    }

private:
    BProgram _bProgram;
    RawSocketManager _rawSocketManager;
    TcpDispatcher _tcpDispatcher;
    TcpSynHandler _tcpSynHandler;
    TcpFinHandler _tcpFinHandler;
    TcpAckSender _tcpAckSender;
    TcpPushSender _tcpPushSender;
    SequenceMaster _sequenceMaster;
    TcpPushHandler _tcpPushHandler;

    BridgeToTcp _bridgeToTcp;
    BridgeToHttp _bridgeToHttp;

    HttpDispatcher _httpDispatcher;
    HttpGetHandler _httpGetHandler;
    HttpErrorHandler _httpErrorHandler;
    HttpFoundHandler _httpFoundHandler;

    Logger _logger;

    void runEngine()
    {
        _bProgram.runEngine();
    }

    void runProgram()
    {
        _bProgram.addThread( _rawSocketManager );
        _bProgram.addThread( _tcpDispatcher );
        _bProgram.addThread( _tcpSynHandler );
        _bProgram.addThread( _tcpFinHandler );
        _bProgram.addThread( _tcpAckSender );
        _bProgram.addThread( _tcpPushSender );
        _bProgram.addThread( _sequenceMaster );
        _bProgram.addThread( _tcpPushHandler );

        _bProgram.addThread( _bridgeToTcp );
        _bProgram.addThread( _bridgeToHttp );

        _bProgram.addThread( _httpDispatcher );
        _bProgram.addThread( _httpGetHandler );
        _bProgram.addThread( _httpErrorHandler );
        _bProgram.addThread( _httpFoundHandler );

        _bProgram.addThread( _logger );

        _bProgram.runProgram();
    }

    void sleepForever()
    {
        enum {
            FOREVER = 10000,
        };

        sleep( FOREVER );
    }

    void reportError( const Error &e )
    {
        printf( "Main Thread: Error caught. Code: %u. Errno: %i\n", e.code(), e.errno() );
        fflush( 0 );
    }
};

#endif // __TcpServer_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
